# -*- coding: utf-8 -*-
"""
Created on Tue Dec  3 14:19:57 2024

@author: Joseph Rosen

initialization file
"""

# Import main functions from submodules
from .mafLoadr import read_maf
from .mafProcessing import classify_variants, merge_mafs, get_titv
from .mafVisualization import get_maf_summary

# Define what should be accessible when the package is imported
__all__ = [
    'read_maf',
    'classify_variants',
    'merge_mafs',
    'get_titv',
    'get_maf_summary'
]

# Optional metadata
__version__ = "0.1"
__author__ = "Joseph Rosen"




# -*- coding: utf-8 -*-
"""
Created on Tue Dec  3 14:33:21 2024

@author: Joseph Rosen

Maf Reader functions file

Contains Functions for loading a maf into a python environment as a dataframe

"""

import pandas as pd

def read_maf(maf_file, skip_rows_until="Hugo_Symbol", sep="\t", use_all= True):
    """
    Reads a MAF file, skipping lines until the specified header is found.
    
    Parameters:
        maf_file (str): Path to the MAF file.
        skip_rows_until (str): Column name to start reading from.
        sep (str): Delimiter used in the MAF file (default is tab).
        
    Returns:
        pd.DataFrame: Loaded MAF file as a DataFrame.
    """
    with open(maf_file, 'r') as f:
        for i, line in enumerate(f):
            if line.startswith(skip_rows_until):
                skip_rows = i
                break
    maf_df = pd.read_csv(maf_file, sep=sep, skiprows=skip_rows, comment="#")
    

    def validate_maf(maf_df, use_all=use_all):
        """
        Validates and filters the MAF DataFrame.
        
        Parameters:
            maf_df (pd.DataFrame): DataFrame containing MAF data.
            use_all (bool): If False, only keep variants with Mutation_Status = 'Somatic'.
            
        Returns:
            pd.DataFrame: Validated MAF DataFrame.
        """
        required_cols = [
            'Hugo_Symbol', 'Chromosome', 'Start_Position', 'End_Position',
            'Reference_Allele', 'Tumor_Seq_Allele2', 'Variant_Classification', 'Tumor_Sample_Barcode', 'Variant_Type'
        ]
        for col in required_cols:
            if col not in maf_df.columns:
                raise ValueError(f"Missing required column: {col}")
        if not use_all and 'Mutation_Status' in maf_df.columns:
            maf_df = maf_df[maf_df['Mutation_Status'] == 'Somatic']
        return maf_df
    maf_df = validate_maf(maf_df, use_all=use_all)
        
    
    return maf_df



# -*- coding: utf-8 -*-
"""
Created on Tue Dec  3 14:42:05 2024

@author: Joseph Rosen

Maf processing functions file:

Contains functions for the processing of maf dataframes    

"""

import pandas as pd



def classify_variants(maf, discard_synonymous= True):
    """
    Separates variants into synonymous and non-synonymous categories.
    """
    non_synonymous = [
        "Frame_Shift_Del", "Frame_Shift_Ins", "Splice_Site", "Translation_Start_Site",
        "Nonsense_Mutation", "Nonstop_Mutation", "In_Frame_Del",
        "In_Frame_Ins", "Missense_Mutation"
    ]
    if discard_synonymous:
        maf = maf[maf['Variant_Classification'].isin(non_synonymous)]
    return maf

def merge_mafs(maf_list, discard_synonymous= False):
    """
    Merges multiple MAF DataFrames into one.

    Parameters:
        maf_list (list): List of MAF DataFrames to merge.

    Returns:
        pd.DataFrame: Merged MAF DataFrame.
    """
    merged_maf = pd.concat(maf_list, axis=0, ignore_index=True)
    merged_maf = merged_maf.drop_duplicates(subset=['Hugo_Symbol', 'Chromosome', 'Start_Position', 'End_Position', 'Tumor_Sample_Barcode'])
    
    if discard_synonymous == True:
        merged_maf = classify_variants(merged_maf, discard_synonymous = True)
        return merged_maf
    
    return merged_maf

def get_titv(maf_df):
    """
    Classify SNVs into Ti/Tv and generate summary for plotting.

    Parameters:
        maf_df (pd.DataFrame): MAF DataFrame containing at least the columns:
                               ['Reference_Allele', 'Tumor_Seq_Allele2', 'Variant_Type'].

    Returns:
        pd.DataFrame: Summary DataFrame with SNV class counts and Ti/Tv classification.
    """
    # Create a copy to avoid SettingWithCopyWarning
    maf_snv_df = maf_df[maf_df['Variant_Type'] == 'SNP'].copy()
    
    # Use .loc for setting new columns
    maf_snv_df.loc[:, 'SNV_Class'] = maf_snv_df['Reference_Allele'] + '>' + maf_snv_df['Tumor_Seq_Allele2']
    
    conv = {
        'A>G': 'T>C', 'T>C': 'T>C',
        'C>T': 'C>T', 'G>A': 'C>T',
        'A>T': 'T>A', 'T>A': 'T>A',
        'A>C': 'T>G', 'T>G': 'T>G',
        'C>A': 'C>A', 'G>T': 'C>A',
        'C>G': 'C>G', 'G>C': 'C>G'
    }
    conv_class = {
        'T>C': 'Ti', 'C>T': 'Ti',
        'T>A': 'Tv', 'T>G': 'Tv',
        'C>A': 'Tv', 'C>G': 'Tv'
    }
    
    # Use .loc for mapping
    maf_snv_df.loc[:, 'SNV_Class'] = maf_snv_df['SNV_Class'].map(conv)
    maf_snv_df.loc[:, 'TiTv'] = maf_snv_df['SNV_Class'].map(conv_class)
    
    maf_snv_df = maf_snv_df.dropna(subset=['SNV_Class', 'TiTv'])

    snv_summary_df = (
        maf_snv_df.groupby('SNV_Class')
        .size()
        .reset_index(name='Raw_Count')
    )
    total_variants = snv_summary_df['Raw_Count'].sum()
    snv_summary_df['Proportion'] = (snv_summary_df['Raw_Count'] / total_variants) * 100
    return snv_summary_df






# -*- coding: utf-8 -*-
"""
Created on Tue Dec  3 14:46:20 2024

@author: Joseph Rosen

Package file containing Visualization functions for mafViper


"""

# Importing dependencies
import seaborn as sns
import matplotlib.pyplot as plt
from matplotlib.backends.backend_pdf import PdfPages
from .mafProcessing import classify_variants, get_titv


  
def get_maf_summary(maf_df, output_pdf: str = None, add_stat: str = None, show_barcodes: bool = False, top_genes: int = 10, raw_count: bool = True, nonsyn_titv: bool = False, use_nonsyn: bool = True):
    if output_pdf is None:
        output_pdf = "MAF_Summary_Dashboard.pdf"
        
    # Summarize Data & filter maf df for non-syn mutations if needed
    if use_nonsyn:
        nonsyn_df = classify_variants(maf_df, discard_synonymous= True)
        maf_data_df = nonsyn_df
    else:
        maf_data_df = maf_df
    
    # Variant classification calculation
    variant_classification = maf_data_df['Variant_Classification'].value_counts().reset_index()
    variant_classification.columns = ['Variant_Classification', 'Count']
    
    gene_summary = maf_data_df.groupby('Hugo_Symbol').size().reset_index(name='Mutation_Count').nlargest(top_genes, 'Mutation_Count')
    variant_type = maf_data_df['Variant_Type'].value_counts().reset_index()
    variant_type.columns = ['Variant_Type', 'Count']
    
    # Setting Data Source based on non-synonymous mutation preference
    # default for titv is synonymous counts- maftools precedent
    if nonsyn_titv: 
        snv_summary_df = get_titv(nonsyn_df)
    else:
        snv_summary_df = get_titv(maf_df)
    
    # Normalize Ti/Tv data if proportions are requested
    if not raw_count:
        SNV_x_col = 'Proportion'
        SNV_x_label = 'Proportion'
    else:
        SNV_x_col = 'Raw_Count'
        SNV_x_label = 'Raw Count'
    
    # Dashboard Plot
    with PdfPages(output_pdf) as pdf:
        fig, axes = plt.subplots(3, 2, figsize=(15, 20)) #3 row 2 col

         # Plot 1: Stacked Bar Chart for Variants Per Sample
        sample_variant_type = maf_data_df.groupby(['Tumor_Sample_Barcode', 'Variant_Type']).size().reset_index(name='Count')
        sample_variant_type_pivot = sample_variant_type.pivot(index='Tumor_Sample_Barcode', columns='Variant_Type', values='Count').fillna(0)

        sample_variant_type_pivot.plot(kind='bar', stacked=True, ax=axes[0, 0], colormap='viridis', edgecolor='none')
        axes[0, 0].set_title('Variants Per Sample', fontsize=12)
        axes[0, 0].set_xlabel('Sample', fontsize=10)
        axes[0, 0].set_ylabel('Mutation Count', fontsize=10)

        # Add mean or median line if specified
        if add_stat:
            if add_stat == 'mean':
                stat_value = sample_variant_type_pivot.sum(axis=1).mean()
            elif add_stat == 'median':
                stat_value = sample_variant_type_pivot.sum(axis=1).median()
            axes[0, 0].axhline(stat_value, color='red', linestyle='--', label=f'{add_stat.capitalize()}: {stat_value:.2f}')
            axes[0, 0].legend()

        # Plot 2: Variant Classification Summary
        sns.barplot(x='Variant_Classification', y='Count', data=variant_classification, ax=axes[0, 1], hue='Variant_Classification', palette='viridis', legend=False)
        axes[0, 1].set_title('Variant Classification Summary')
        axes[0, 1].set_xlabel('Variant Classification')
        axes[0, 1].set_ylabel('Count')
        axes[0, 1].tick_params(axis='x', rotation=45)

        # Plot 3: Top Mutated Genes
        sns.barplot(y='Hugo_Symbol', x='Mutation_Count', data=gene_summary, ax=axes[1, 0], hue='Hugo_Symbol', palette='viridis', legend=False)
        axes[1, 0].set_title(f'Top {top_genes} Mutated Genes')
        axes[1, 0].set_xlabel('Mutation Count')
        axes[1, 0].set_ylabel('Gene')
        axes[1, 0].set_xlim(0, 4)

        # Plot 4: Boxplot for Variant Classification
        sns.boxplot(x='Variant_Classification', y='t_alt_count', data=maf_data_df, ax=axes[1, 1], hue='Variant_Classification', palette='viridis', legend=False)
        axes[1, 1].set_title('Boxplot of Variant Classification Counts')
        axes[1, 1].set_xlabel('Variant Classification')
        axes[1, 1].set_ylabel('Alternate Allele Counts')
        axes[1, 1].tick_params(axis='x', rotation=45)
        
        # Plot 5: SNV Class Summary (Ti/Tv Data)
        if not raw_count:
            sns.barplot(data=snv_summary_df, x=SNV_x_col, y='SNV_Class', ax=axes[2, 0], hue= 'SNV_Class', palette='viridis', orient='h')
            axes[2, 0].set_xlim(0, 100)  # Set x-axis limit to percentage
            for i, value in enumerate(snv_summary_df['Proportion']):
                axes[2, 0].text(value + 1, i, f"{snv_summary_df['Raw_Count'].iloc[i]}", va='center', fontsize=9)
        else:
            sns.barplot(data=snv_summary_df, x=SNV_x_col, y='SNV_Class', ax=axes[2, 0], palette='viridis', orient='h')
        
        axes[2, 0].set_title('SNV Class Summary')
        axes[2, 0].set_xlabel(SNV_x_label, fontsize=10)
        axes[2, 0].set_ylabel('SNV Class')

        # Plot 6: Variant Type Summary (e.g., SNP, INS, DEL)
        sns.barplot(x='Variant_Type', y='Count', data=variant_type, ax=axes[2, 1], hue='Variant_Type', palette='viridis',  legend=False)
        axes[2, 1].set_title('Variant Type Summary')
        axes[2, 1].set_xlabel('Variant Type')
        axes[2, 1].set_ylabel('Count')
      
        # Tight layout and save PDF
        plt.tight_layout(rect=[0, 0.03, 1, 0.95])
        plt.show()
        pdf.savefig(fig)
        plt.close(fig)

    print(f"Summary dashboard saved to: {output_pdf}")


# -*- coding: utf-8 -*-
"""
Created on Tue Dec  3 14:57:31 2024

@author: Joseph Rosen


"""

from setuptools import setup, find_packages

with open('README.md', 'r') as f:
    description=f.read()

setup(
    name="mafViper",  # Package name
    version="0.2",  # Better Readme
    author="Joseph Rosen",
    author_email="joemrosen@gmail.com",
    description="A Python package for processing, analyzing and visualizing MAF files.",
    packages=find_packages(exclude=["tests", "prototypes", "*.pdf", "*.maf", "*.csv", "*.xlsx" ]),
    long_description=description,
    long_description_content_type="text/markdown",
    url="https://github.com/jrosen-code/mafViper",  
    python_requires=">=3.9",  # Minimum Python version
    install_requires=[
        "pandas>=2.2.2",
        "seaborn>=0.13.2",
        "matplotlib>=3.9.2",
    ],
    include_package_data=True,  # Ensures non-code files like README are included
)

